/* sm_func.c: This file contains the Solarmax functions for the Solarmax proxy.
 * Version: 0.1
 * Authors: Axel C. Voigt
 * Date: October 26, 2009
 */

/* standard includes */
#include "sm_func.h"

extern int debug;
extern FILE* logfp;

/* ------------------------------------------------------------------------- */
/* sm_PrepareAnswer                                                          */
/* ------------------------------------------------------------------------- */
char* sm_PrepareAnswer(int num, unsigned char* invadr, unsigned char* masteradr, 
        solarmax_data* smdata, unsigned char* answer)
{
	unsigned int z= 0;
	int cs= 0;
	int len= 0;
	unsigned char tmp[255];

	strcpy(tmp, "64:");
	sprintf(&tmp[strlen(tmp)], "DYR=%X", smdata[num-1].dyr);
	sprintf(&tmp[strlen(tmp)], ";DMT=%X", smdata[num-1].dmt);
	sprintf(&tmp[strlen(tmp)], ";DDY=%X", smdata[num-1].ddy);
	sprintf(&tmp[strlen(tmp)], ";PAC=%X", (int)(smdata[num-1].pac/100*2)); /*0.5*/
	sprintf(&tmp[strlen(tmp)], ";PP=%X", smdata[num-1].pp*2); /*0.5*/
	sprintf(&tmp[strlen(tmp)], ";KYR=%X", (int)smdata[num-1].kyr);
	sprintf(&tmp[strlen(tmp)], ";KMT=%X", (int)smdata[num-1].kmt);
	sprintf(&tmp[strlen(tmp)], ";KDY=%X", (int)(smdata[num-1].kdy*10));
	sprintf(&tmp[strlen(tmp)], ";KT0=%X", (int)smdata[num-1].kt0);
	sprintf(&tmp[strlen(tmp)], ";TYP=%X", (int)smdata[num-1].typ);
	sprintf(&tmp[strlen(tmp)], ";UDC=%X", (int)(smdata[num-1].udc/10));
	sprintf(&tmp[strlen(tmp)], ";UL1=%X", (int)(smdata[num-1].ul1/10));
	sprintf(&tmp[strlen(tmp)], ";IDC=%X", (int)smdata[num-1].idc);
	sprintf(&tmp[strlen(tmp)], ";IL1=%X", (int)smdata[num-1].il1);
	// sprintf(&tmp[strlen(tmp)], ";TKK=%x", smdata[num-1].tkk);
	// sprintf(&tmp[strlen(tmp)], ";SWV=%x", smdata[num-1].swv);
	sprintf(&tmp[strlen(tmp)], ";SYS=%X", (int)smdata[num-1].sys);
	// sprintf(&tmp[strlen(tmp)], ";SAL=%x", smdata[num-1].sal);

	len= strlen(tmp); /* length of data */
	sprintf(&tmp[strlen(tmp)], "|");

	/* build answer */
	sprintf( answer, "{%s;%s;%X|%s", invadr, masteradr, len, tmp );

	/* create checksum */
	for(z=1;z<strlen(answer);z++)
	{
		cs+=answer[z];
	}

	/* finalize answer */
	sprintf(&answer[strlen(answer)], "%04X}", cs);

	return answer;
}
/* ----- sm_PrepareAnswer -------------------------------------------------- */

/* ------------------------------------------------------------------------- */
/* sm_CheckCommand                                                           */
/* ------------------------------------------------------------------------- */
int sm_CheckCommand(Socket* skt, unsigned char* invadr, unsigned char* masteradr, 
      unsigned char* msglen, unsigned char* data)
{
	unsigned char tmp[255];
	int len= 0;
	unsigned int z= 0;
	int cs= 0;
	int cs_calc= 0;
	struct timeval tv;
	unsigned char* p= NULL;

	if(debug==1) { fprintf(logfp, "smp - CheckCommand() waiting for data ...\n"); fflush(logfp); }
	if(0>Stimeoutwait(skt, 1, 0)) /* wait for data ... */
	{
		fprintf(logfp, "smp - ERROR - connection detected but no data (timeout:%dsec)\n", 1);
		fflush(logfp);
		return -1;
	}

	if(debug==1) { fprintf(logfp, "smp - CheckCommand() reading data ...\n"); fflush(logfp); }
	len= Sread(skt, tmp, 255);
	if(len<=0)
	{
		fprintf(logfp, "smp - ERROR - read returned no data (%d)!\n", len);
		fflush(logfp);
		return -2;
	}

	tmp[len>255?254:len]='\0';
	//strcpy(tmp, COMMAND);
	if(debug==1) // PeMue time added
	{
		gettimeofday (&tv, NULL);
		fprintf(logfp, "smp - %ld.%d CheckCommand() received \n>%s< (%d bytes)\n", tv.tv_sec, tv.tv_usec, tmp, len); 
		fflush(logfp);
	}

	p= strtok(tmp, "{;");
	if(!p) return -3;
	strcpy(masteradr, p);
	p=strtok(NULL, ";|}");
	if(!p) return -4;
	strcpy(invadr, p);
	p=strtok(NULL, "|}");
	if(!p) return -5;
	strcpy(msglen, p);
	p=strtok(NULL, "|");
	if(!p) return -6;
	strcpy(data, p);
	p=strtok(NULL, "}\0");
	if(!p) return -7;
	sscanf(p, "%X", &cs);;

	sprintf(tmp,"{%s;%s;%s|%s|", masteradr, invadr, msglen, data);

	/* calculate checksum */
	cs_calc= 0;
	for(z=1;z<strlen(tmp);z++)
	{
		cs_calc+=tmp[z];
	}
	if(debug==1) { fprintf(logfp, "smp - CheckCommand() checksum %s\n", cs==cs_calc?"OK":"ERR"); fflush(logfp); }
	return (cs==cs_calc?0:-8);
}
/* ----- sm_CheckCommand --------------------------------------------------- */
