/* process_ids.c: This file checks the process IDs, part of the Solarmax proxy.
 * Version: 0.1
 * Authors: Axel C. Voigt
 * Date: October 26, 2009
 */

#include <stdio.h>
#include <string.h>
#include <dirent.h>
#include <regex.h>
#include <sys/types.h>
#include <unistd.h>

/* ------------------------------------------------------------------------- */
/* list_process_ids                                                          */
/* ------------------------------------------------------------------------- */
int list_process_ids (const char *name)
{
	DIR *dir;
	struct dirent *dir_info;
	regex_t preq;
	char filename[64], buffer[256];
	FILE *fz;
	int len, count = 0;

	// compile regular expressions
	regcomp (&preq, "^[[:digit:]]+$", REG_EXTENDED);

	//open proc directory
	dir = opendir ("/proc");
	if (dir)
	{
		// open all process status files and get name of process
		while ((dir_info = readdir (dir)))
		{
			if (!regexec (&preq, dir_info->d_name, 0, NULL, 0))
			{
				memset (filename, 0, 64);
				sprintf (filename, "/proc/%s/status", dir_info->d_name);

				if ((fz = fopen (filename, "r")))
				{
					// read first line of file and check name of process
					memset (buffer, 0, 256);
					if ((fgets (buffer, 256, fz)))
					{
						len = strlen (buffer);
						buffer[len - 1] = '\0';

						if (!strcmp (buffer + 6, name))
						{
							char tmp[10];
							sprintf(tmp, "%d", getpid());
							
							/* is it myself that I am looking at? */
							if(0!=strcmp(tmp, dir_info->d_name))
								++count;
						}
					}
					// close file
					fclose (fz);
				}
			}
		}

		// close directory
		closedir (dir);

		if (!count)
		{
			return 0;
		}
		else
		{
			//fprintf (stdout, "\n");
			return count;
		}
	}
	else
	{
		fprintf (stderr, "Couldn't open /proc\n\n");
		return -1;
	}

	// free memory
	regfree (&preq);
}
/* ----- list_process_ids -------------------------------------------------- */