import serial
import time
import sys

s = serial.Serial(
    port='/dev/ttyUSB0',
    baudrate=9600,
    parity=serial.PARITY_NONE,
    bytesize=serial.EIGHTBITS,
    stopbits=serial.STOPBITS_ONE,
    timeout=5
)

def get_data(address, command, block):
    checksum = address + command + block
    request = [0, address, command, block, checksum]
    s.write(bytearray(request))
    s.flush()
    print("Sending request: {0}".format(request))
    needed_bytes = 13
    if command == 8:
        needed_bytes = 24
    answer = s.read_until(size=needed_bytes)
    print("Answer received: {0}".format(answer))
    pretty_print(answer)
    return answer

def pretty_print(answer):
    output = [int(x) for x in answer]
    print("Pretty: {0} len={1}".format(output,len(output)))

def validate_checksum(answer):
    length = len(answer)
    sum = 0
    for i in range(0, length-1):
#        print("Adding {0}".format(i))
        sum += answer[i]
    given_checksum = answer[length-1]
    calculated_checksum = sum % 256
    print("Given checksum: {0}, Calculcated checksum: {1}".format(given_checksum, calculated_checksum))
    return given_checksum == calculated_checksum

def parseOnlineValues(answer1):
    answer = [chr(x) for x in answer1]

    udc = ord(answer[0]) * 2.8 +100
    idc = ord(answer[1]) * 0.08
    uac = ord(answer[2]) + 100.0
    iac = ord(answer[3]) * 0.120
    t = ord(answer[4]) - 40.0
    s = ord(answer[5]) * 6.0
    pdc = int(udc * idc) / 1000
    pac = int(uac * iac) / 1000
    wd = (ord(answer[6]) * 256 + ord(answer[7])) / 1000
    wtot = ord(answer[8]) * 256 + ord(answer[9])
    print("UDC: {0}, IDC: {1}, UAC: {2}, IAC: {3}, T: {4}, S: {5}, PDC: {6}, PAC: {7}, WD: {8}, WTOT: {9}".format(udc,idc,uac,iac,t,s,pdc,pac,wd,wtot))

for i in range(2,3):
    answer = get_data(1,i,1)
    validate_checksum(answer)
    parseOnlineValues(answer)
    print("----------------------------------------------------------------------------------------------------")


#for i in range(1,2):
#    command = 3
#    hash = i + command + 1
#    cmd = [0, i, command, 1, hash]
#    print("Send command {0}".format(cmd))
#    cmdBytes = bytearray(cmd)
#    s.write(cmdBytes)
#    s.flush
#    byte_values= s.read_until(size=13)
#    print("Byte values: {0}".format(byte_values))
#    b1 = byte_values[0]
#    b2 = byte_values[1]
#    b3 = byte_values[2]
#    b4 = byte_values[3]
#    b5 = byte_values[4]
#    b6 = byte_values[5]
#    b7 = byte_values[6]
#    b8 = byte_values[7]
#    b9 = byte_values[8]
#    b10= byte_values[9]
#    b11= byte_values[10]
#    b12= byte_values[11]
#    cs = byte_values[12]
#    print("[{0},{1},{2},{3},{4},{5},{6},{7},{8},{9},{10},{11}] cs: {12}".format(b1,b2,b3,b4,b5,b6,b7,b8,b9,b10,b11,b12,cs))
#    calculated_checksum = (b1+b2+b3+b4+b5+b6+b7+b8+b9+b10+b11+b12) % 256
#    print("Checksum: {0}".format(calculated_checksum))



#cmd = [0, 2, 3, 1, 6]
#cmdBytes = bytearray(cmd)
#s.write(cmdBytes)
#s.flush()

#while 1:
#    print(s.read(13))
